#! /bin/bash
PID=$$
FIFO=/tmp/FIFO${PID}
mkfifo ${FIFO}
######################################################################################################

#############################
##### Code ffmpeg final #####
#############################
function code_final
{
unset lock

# Début de ffmpeg
code="ffmpeg"

# En cas de parametrage de durée pre enregistrement
if [[ ${delay_heure} -gt 0 || ${delay_minute} -gt 0 || ${delay_seconde} -gt 0 ]]
then
	code+=" -ss \""
	[[ ${delay_heure} == [0-9] ]] && code+="0${delay_heure}:" || code+="${delay_heure:-00}:"
	[[ ${delay_minute} == [0-9] ]] && code+="0${delay_minute}:" || code+="${delay_minute:-00}:"
	[[ ${delay_seconde} == [0-9] ]] && code+="0${delay_seconde}\"" || code+="${delay_seconde:-00}\""
fi

# En cas de parametrage de durée d'enregistrement
if [[ ${capture_heure} -gt 0 || ${capture_minute} -gt 0 || ${capture_seconde} -gt 0 ]]
then
	code+=" -t \""
	[[ ${capture_heure} == [0-9] ]] && code+="0${capture_heure}:" || code+="${capture_heure:-00}:"
	[[ ${capture_minute} == [0-9] ]] && code+="0${capture_minute}:" || code+="${capture_minute:-00}:"
	[[ ${capture_seconde} == [0-9] ]] && code+="0${capture_seconde}\"" || code+="${capture_seconde:-00}\""
fi

# Si on capture l'audio, on commence par ça, traiter tout l'audio directement semble poser probleme
if [[ ${audio_no} != True ]]
then
	code+=" -f \"${audio_serveur:-alsa}\" -i \"${audio_provenance:-pulse}\""
fi

# Si on capture la vidéo, on commence par ça
if [[ ${video_no} != True ]]
then
	code+=" -f \"x11grab\""
	[[ ${video_fps} -gt 0 ]] && code+=" -r \"${video_fps}\""
	code+=" -s \"${video_reso_l_in}x${video_reso_h_in}\" -i \"${sortie_display}+${sortie_emplacement}\""
fi

# Si on capture l'audio, on continue avec ça
if [[ ${audio_no} != True ]]
then
	if [[ ${video_no} != True ]]
	then
		case ${video_codec} in
			libx264) code+=" -acodec \"${audio_codec:-libvorbis}\"" ;;
			libvpx) code+=" -acodec \"libvorbis\"" ;;
			flv) code+=" -acodec \"libmp3lame\"" ;;
		esac

	else
		code+=" -acodec \"${audio_codec:-libvorbis}\""
	fi

	[[ ${audio_codec} == aac ]] && code+=" -strict \"experimental\""
	[[ ${audio_frequence} -gt 0 ]] && code+=" -ar \"${audio_frequence}\""
	[[ ${audio_bitrate} -gt 0 ]] && code+=" -ab \"${audio_bitrate}k\""
fi

# Si on capture la vidéo, on continue avec ça
if [[ ${video_no} != True ]]
then
	code+=" -vcodec ${video_codec}"

	# Options du codec x264
	if [[ ${video_codec} == "libx264" ]]
	then
		[[ ${x264_crf} -gt 0 ]] && code+=" -crf \"${x264_crf}\""
		code+=" -vpre \"${x264_vpre:-lossless_ultrafast}\""
		[[ ${x264_nocabac:-False} == True ]] && code+=" -coder \"0\"" || code+=" -coder \"1\""
		[[ ${x264_algorithme} -gt 0 ]] && code+=" -subq \"${x264_algorithme}\""
	fi

	# Options incompatibles avec flash
	if [[ ${video_codec} != "flv" ]]
	then
		code+=" -dia_size ${video_diasize:-hex}"
		[[ ${video_bframe} -gt 0 ]] && code+=" -bf \"${video_bframe}\""
		[[ ${video_references} -gt 0 ]] && code+=" -refs \"${video_references}\""
		[[ ${video_multithreads} ]] && code+=" -threads \"${video_multithreads}\""
	else
		code+=" -dia_size \"${video_diasize:-hex}\""
	fi

	# Options en fonction du type de debit
	if [[ ${video_debit} == "fixe" ]]
	then
		[[ ${video_bitrate} -gt 0 ]] && code+=" -b \"${video_bitrate}k\""
		[[ ${video_tolerance} -gt 0 ]] && code+=" -bt \"${video_tolerance}k\""

	elif [[ ${video_debit} == "variable" ]]
	then
		[[ ${video_qmin} -gt 0 ]] && code+=" -qmin \"${video_qmin}\""
		[[ ${video_qmax} -gt 0 ]] && code+=" -qmax \"${video_qmax}\""
	fi

	[[ ${video_framekey} -gt 0 ]] && code+=" -g \"${video_framekey}\""
	[[ ${video_framekey_min} -gt 0 ]] && code+=" -keyint_min \"${video_framekey_min}\""
	[[ ${video_deblockalpha} -ne 0 ]] && code+=" -deblockalpha \"${video_deblockalpha}\""
	[[ ${video_deblockbeta} -ne 0 ]] && code+=" -deblockbeta \"${video_deblockbeta}\""
	[[ ${video_treillis} -gt 0 ]] && code+=" -trellis \"${video_treillis}\""
	[[ ${video_desentralacement} == True ]] && code+=" -deinterlace"
	[[ ${video_gris} == True ]] && code+=" -flags \"gray\""
fi

# Si on capture la vidéo, on continue avec ça
[[ ${video_no} != True ]] && code+=" -r \"${sortie_fps:-25}\" -s \"${video_reso_l_out}x${video_reso_h_out}\""

# Adresse de sortie du fichier
[[ ${video_no} != True ]] && sortie_fichier="${sortie_dossier}/Screencastor_$(date +'%s').${ext}" || sortie_fichier="${sortie_dossier}/Screencastor_$(date +'%s').${sortie_extension_audio:-ogg}"
code+=" \"${sortie_fichier}\""

# Nettoyage du textview
echo 'TEXT@@CLEAR@@_code_ffmpeg'

# Mini attente pour éviter tout probleme entre les 2 commandes
sleep 0.025

# Ajout de la ligne de code ffmpg dans le textview
echo "TEXT@@END@@_code_ffmpeg@@${code}"

# Affichage de la fenetre
[[ ${visible} == False ]] && { echo 'SET@screencastor.show()'; unset visible; }
}


#########################
##### Systeme video #####
#########################
function _video_no
{
video_no=${@}

# Modification des extentions
if [[ ${video_no} == True ]]
then
	echo 'SET@_video_frame.hide()'
	echo 'SET@_audio_frame.show()'
	[[ ${lock} != True ]] && code_final

else
	echo 'SET@_video_frame.show()'
	echo 'SET@_audio_frame.hide()'

	echo "COMBO@@FINDSELECT@@_sortie_extension_video@@${ext}"
fi
}

function _x264
{
# En cas de reload post modification des resolutions
if [[ -z ${lock} && ${ext} ]] || [[ ${lock} && -z ${ext} ]] || [[ -z ${lock} && -z ${ext} ]]
then
	video_codec=libx264
	ext=mkv
fi

# Si pas de video ou si ce bouton est en False, on stoppe la fonction
[[ ${video_no} == True || ${1} == False ]] && return

# Modifications graphiques
echo 'MULTI@@SET@@set_sensitive(True)@@_x264_nocabac,_video_multithreads,_x264_vpre,_x264_crf,audio_codec'
echo 'MULTI@@SET@@set_sensitive(False)@@_webm_level'
echo "COMBO@@FINDSELECT@@_sortie_extension_video@@${ext}"
}

function _webm
{
# En cas de reload post modification des resolutions
if [[ -z ${lock} && ${ext} ]] || [[ ${lock} && -z ${ext} ]] || [[ -z ${lock} && -z ${ext} ]]
then
	video_codec=libvpx
	ext=webm
fi

# Si pas de video ou si ce bouton est en False, on stoppe la fonction
[[ ${video_no} == True || ${1} == False ]] && return

# Si la version ffmpeg est trop ancienne, on stoppe
if [[ $(apt-cache show ffmpeg | sed -n "/^Version/ s/.*:\(.*\)-.*/\1/p") < 0.6.0 ]]
then
	# Nettoyage du textview
	echo 'TEXT@@CLEAR@@_code_ffmpeg'

	# Mini attente pour éviter tout probleme entre les 2 commandes
	sleep 0.025

	# Ajout de la ligne de code ffmpg dans le textview
	echo "TEXT@@END@@_code_ffmpeg@@$(eval_gettext $'The FFmpeg version is too old (< 0.6.0) and does not handle the Webm codec.')"

else
	# Modifications graphiques
	echo 'MULTI@@SET@@set_sensitive(True)@@_video_multithreads,_webm_level'
	echo 'MULTI@@SET@@set_sensitive(False)@@_x264_nocabac,_x264_vpre,_x264_crf'
	echo 'SET@audio_codec.set_sensitive(False)'
	echo "COMBO@@FINDSELECT@@_sortie_extension_video@@${ext}"
fi
}

function _flash
{
# En cas de reload post modification des resolutions
if [[ -z ${lock} && ${ext} ]] || [[ ${lock} && -z ${ext} ]] || [[ -z ${lock} && -z ${ext} ]]
then
	video_codec=flv
	ext=flv
fi

# Si pas de video ou si ce bouton est en False, on stoppe la fonction
[[ ${video_no} == True || ${1} == False ]] && return

# Modifications graphiques
echo 'MULTI@@SET@@set_sensitive(False)@@_x264_nocabac,_x264_vpre,_x264_crf,_video_multithreads,_video_minrate,_video_maxrate_webm_level'
echo 'SET@audio_codec.set_sensitive(False)'
echo "COMBO@@FINDSELECT@@_sortie_extension_video@@${ext}"
}

##### Video #####
function _video_reso_l_out { video_reso_l_out=${@%%.*}; [[ ${lock} != True ]] && code_final; }

function _video_reso_h_out { video_reso_h_out=${@%%.*}; [[ ${lock} != True ]] && code_final; }

function _video_reso_l_in { video_reso_l_in=${@%%.*}; [[ ${lock} != True ]] && code_final; }

function _video_reso_h_in { video_reso_h_in=${@%%.*}; [[ ${lock} != True ]] && code_final; }

function _video_bitrate { video_bitrate=${@%%.*}; [[ ${lock} != True ]] && code_final; }

function _video_tolerance { video_tolerance=${@%%.*}; [[ ${lock} != True ]] && code_final; }

function _video_fps { video_fps=${@%%.*}; [[ ${lock} != True ]] && code_final; }

function _video_qmin { video_qmin=${@%%.*}; [[ ${lock} != True ]] && code_final; }

function _video_qmax { video_qmax=${@%%.*}; [[ ${lock} != True ]] && code_final; }

function _video_framekey { video_framekey=${@%%.*}; [[ ${lock} != True ]] && code_final; }

function _video_framekey_min { video_framekey_min=${@%%.*}; [[ ${lock} != True ]] && code_final; }

function _video_deblockalpha { video_deblockalpha=${@%%.*}; [[ ${lock} != True ]] && code_final; }

function _video_deblockbeta { video_deblockbeta=${@%%.*}; [[ ${lock} != True ]] && code_final; }

function _video_references { video_references=${@%%.*}; [[ ${lock} != True ]] && code_final; }

function _video_treillis { video_treillis=${@%%.*}; [[ ${lock} != True ]] && code_final; }

function _video_bframe { video_bframe=${@%%.*}; [[ ${lock} != True ]] && code_final; }

function _video_lookahead { video_lookahead=${@%%.*}; [[ ${lock} != True ]] && code_final; }

function _video_desentralacement { video_desentralacement=${@}; [[ ${lock} != True ]] && code_final; }

function _video_multithreads { video_multithreads=${@%%.*}; [[ ${lock} != True ]] && code_final; }

function _video_gris { video_gris=${@}; [[ ${lock} != True ]] && code_final; }

function _debit_variable { [[ ${1} == True ]] && { video_debit=variable; [[ ${lock} != True ]] && code_final; }; }

function _debit_fixe { [[ ${1} == True ]] && { video_debit=fixe; [[ ${lock} != True ]] && code_final; }; }

function _video_diasize { video_diasize=${1}; [[ ${lock} != True ]] && code_final; }

##### Video x264 #####
function _x264_vpre { x264_vpre=${@}; [[ ${lock} != True ]] && code_final; }

function _x264_crf { x264_crf=${@%%.*}; [[ ${lock} != True ]] && code_final; }

function _x264_nocabac { x264_nocabac=${@}; [[ ${lock} != True ]] && code_final; }

function _x264_algorithme { x264_algorithme=${@%%.*}; [[ ${lock} != True ]] && code_final; }

##### Video webm #####
function _webm_level { webm_level=${1}; [[ ${lock} != True ]] && code_final; }


##### Resolutions #####
function win_info
{
# $1 : 800 : largeur
# $2 : 600 : hauteur
# $3 : 25 : x
# $4 : 25 : y

# Vérification que la largeur/hauteur est un multiple de 2, sinon ajout d'1 px
[[ $((${1} % 2)) -ne 0 ]] && largeur=$((${1} + 1))
[[ $((${2} % 2)) -ne 0 ]] && hauteur=$((${2} + 1))

# Blocage du lancement des code_final
lock=True

# Envoie des infos
echo "MULTI@@SET@@set_value(${largeur:-${1}})@@_video_reso_l_out,_video_reso_l_in"
echo "MULTI@@SET@@set_value(${hauteur:-${2}})@@_video_reso_h_out,_video_reso_h_in"
echo "SET@_sortie_emplacement.set_text('${3},${4}')"

# Chargement de l'onglet lancant code_final (pour eviter qu'il ne soit charger 4 fois)
case ${video_codec} in
	libvpx) echo 'SET@_webm.set_active(True)'; echo 'SET@_webm.toggled()' ;;
	flv) echo 'SET@_flash.set_active(True)'; echo 'SET@_flash.toggled()' ;;
	libx264|*) echo 'SET@_x264.set_active(True)'; echo 'SET@_x264.toggled()' ;;
esac

# Destruction des variables
unset infos largeur hauteur
}

function sortie_actuelle
{
# Retour aux valeurs de base
win_info ${win_largeur} ${win_hauteur} 0 0
}

function sortie_fenetre
{
# Récupération des infos d'emplacement de la fenetre et de sa résolution avec xwininfo
infos=$(xwininfo) infos=${infos##*-geometry } infos=${infos//x/ } infos=${infos//+/ } infos=${infos//-/ }

# Traitement des infos
win_info ${infos}
}

function sortie_portion
{
# Selection à main lever de l'emplacement
infos=$(import -identify /dev/null) infos=${infos//x/ } infos=(${infos//+/ })

# Vérifie qu'il n'y a pas juste eu un clic
if [[ ! ${infos[6]} =~ ^[[:digit:]]+$ || ! ${infos[7]} =~ ^[[:digit:]]+$ ]]
then
	# Information sur l'echec de l'action
	echo "STATUS@@_statusbar@@$(eval_gettext $'You must make a selection, not a click.')"

	# Arret de la fonction
	return
fi

# Traitement des infos
win_info ${infos[2]} ${infos[3]} ${infos[6]} ${infos[7]}
}


#########################
##### Systeme audio #####
#########################
function audio_liste
{
# Rechargement des codecs audio de ffmpeg en fonction de sa version
if [[ $(ffmpeg -version 2>/dev/null | sed -n "s/^[Ff][Ff]mpeg.*:\(.*\)-.*/\1/p") < 0.6.2 ]]
then
	ffmpeg -formats 2>/dev/null | egrep "^ .EA" | sed " s/   */###/g ; s/.*###\(.*\)###\(.*\)/\1 | \2/ ; /libopencore/d; /^pcm_[^s]/d ; /^pcm_s..[^l]/d ; /^adpcm/d ; /^sonic/d ; /^$/d" > ffmpeg_audio

else
	ffmpeg -codecs 2>/dev/null | egrep "^ .EA" | sed "s/   */###/g ; s/.*###\(.*\)###\(.*\)/\1 | \2/ ; /^pcm_[^s]/d ; /^pcm_s..[^l]/d ; /^adpcm/d ; /^sonic/d ; /^$/d" > "${ffmpeg_audio}"
fi

# Rechargement du tree des codecs audio
echo "TREE@@CLEAR@@audio_codec"
sleep 0.1
echo "TREE@@LOAD@@audio_codec@@${ffmpeg_audio}"
}


function _audio_no
{
audio_no=${@}

[[ ${lock} != True ]] && code_final
}

function audio_codec
{
audio_codec=${1#*@}

case ${audio_codec} in
	aac) echo 'COMBO@@FINDSELECT@@_sortie_extension_audio@@aac' ;;
	ac3*) echo 'COMBO@@FINDSELECT@@_sortie_extension_audio@@ac3' ;;
	flac) echo 'COMBO@@FINDSELECT@@_sortie_extension_audio@@flac' ;;
	libvorbis|vorbis) echo 'COMBO@@FINDSELECT@@_sortie_extension_audio@@ogg' ;;
	libmp3lame) echo 'COMBO@@FINDSELECT@@_sortie_extension_audio@@mp3' ;;
	pcm*) echo 'COMBO@@FINDSELECT@@_sortie_extension_audio@@wav' ;;
	wmav*) echo 'COMBO@@FINDSELECT@@_sortie_extension_audio@@wma' ;;
esac

[[ ${lock} != True ]] && code_final
}

function _audio_frequence { audio_frequence=${@}; [[ ${lock} != True ]] && code_final; }

function _audio_bitrate { audio_bitrate=${@}; [[ ${lock} != True ]] && code_final;  }

function _audio_provenance { audio_provenance=${@}; [[ ${lock} != True ]] && code_final; }

function _audio_serveur { audio_serveur=${@}; [[ ${lock} != True ]] && code_final; }


###########################
##### Systeme options #####
###########################
function statuinfo
{
# 1 : nom de la fonction, 2 true ou false

if [[ ${2} == True ]]
then
	# Blocage du bouton d'enregistrement
	echo 'SET@capture.set_sensitive(False)'

	# Nom de la fonction bloquant le bouton
	blocage+=( ${1} )

	# Envoie d'une info dans la statusbar
	case ${1} in
		sortie_display) echo "STATUS@@_statusbar@@$(eval_gettext $'Invalid location value! Value must be X,X')";;

		sortie_emplacement) echo "STATUS@@_statusbar@@$(eval_gettext $'Invalid screen value! Value must be :X.X')";;
	esac

elif [[ ${2} == False ]]
then
	# Ablation du nom de la fonction bloquant le bouton
	blocage=( ${blocage[@]//${1}} )

	# Si la variable est vide
	if [[ -z ${blocage[@]} ]]
	then
		# Reactive le bouton de capture
		echo 'SET@capture.set_sensitive(True)'

		# Efface le texte de la statusbar
		echo 'STATUS@@_statusbar@@'
	fi
fi
}

function _sortie_display
{
# Vérification de la valeur
if [[ ! "${@}" =~ ^:[0-9]+\.[0-9]+$ ]]
then
	# Blocage du bouton d'enregistrement
	statuinfo sortie_display True

else
	# Deblocage du bouton d'enregistrement
	statuinfo sortie_display False

	sortie_display=${@}
	[[ ${lock} != True ]] && code_final
fi
}

function _sortie_emplacement
{
# Vérification de la valeur
if [[ ! "${@}" =~ ^[0-9]+,[0-9]+$ ]]
then
	# Blocage du bouton d'enregistrement
	statuinfo sortie_emplacement True

else
	# Deblocage du bouton d'enregistrement
	statuinfo sortie_emplacement False

	sortie_emplacement=${@}
	[[ ${lock} != True ]] && code_final
fi
}

function _sortie_dossier
{
# Si la valeur est vide ou identique à l'ancienne, on quitte la fonction
[[ -z ${sortie_dossier} || ${sortie_dossier} == ${@} ]] && return

sortie_dossier=${@}
[[ ${lock} != True ]] && code_final
}

function _sortie_fps { sortie_fps=${@%%.*}; [[ ${lock} != True ]] && code_final; }

function _sortie_extension_audio { sortie_extension_audio=${@}; [[ ${lock} != True ]] && code_final; }

function _sortie_extension_video { ext=${@}; code_final; }

function _capture_heure { capture_heure=${@%%.*}; [[ ${lock} != True ]] && code_final; }

function _capture_minute { capture_minute=${@%%.*}; [[ ${lock} != True ]] && code_final; }

function _capture_seconde { capture_seconde=${@%%.*}; [[ ${lock} != True ]] && code_final; }

function _delay_heure { delay_heure=${@%%.*}; [[ ${lock} != True ]] && code_final; }

function _delay_minute { delay_minute=${@%%.*}; [[ ${lock} != True ]] && code_final; }

function _delay_seconde { delay_seconde=${@%%.*}; [[ ${lock} != True ]] && code_final; }

function save_geo
{
# Récupération des infos d'emplacement de la fenetre et de sa résolution avec xwininfo
geo_fenetre=$(xwininfo -name "Screencastor v${version} - by Hizoka") geo_fenetre=${geo_fenetre##*-geometry } geo_fenetre=${geo_fenetre//x/ } geo_fenetre=${geo_fenetre//+/ } geo_fenetre=${geo_fenetre//-/ }

sed -i "s/geo_fenetre=.*/geo_fenetre=\"${geo_fenetre}\"/" "${fichier_pref}"
}

###########################
##### Boutons actions #####
###########################
function capture
{
# Si besoin, recuperation du numero de l'onglet courant
[[ ${option_onglet} == True ]] && { echo 'GET@_notebook1.get_current_page()'; echo 'ITER@@onglet'; }

echo 'TEXT@@HIZO@@_code_ffmpeg'
}

function _code_ffmpeg
{
# Récupération du code
ffmepg_code=${@#hizo?} ffmepg_code=${ffmepg_code%@@}

if [[ ${ffmpeg_isRunnig} == True ]]
then
	# nouvelle instance demandé, arrêt de l'ancienne et modif de la variable
	flag_new_instance=True

	kill -15 ${ffmpeg_pid}
else
	flag_new_instance=False

	# Modification du numero du fichier pour être a jour
	[[ ${video_no} != True ]] && sortie_fichier="${sortie_dossier}/Screencastor_$(date +'%s').${ext}" || sortie_fichier="${sortie_dossier}/Screencastor_$(date +'%s').${sortie_extension_audio:-ogg}"
	ffmepg_code=$(sed "s@\(.*\) \"/.*@\1 \"${sortie_fichier}\"@" <<< "${ffmepg_code}")

	# Affichage de l'onglet terminal
	echo 'SET@_notebook1.set_current_page(3)'

	# Modifications graphiques
	echo 'SET@_menuitem_record.set_sensitive(False)'
	echo 'SET@_menuitem_stop.set_sensitive(True)'
	echo 'SET@arret.show()'
	echo 'SET@capture.hide()'
	[[ ${systray_auto} == True ]] && echo 'SET@screencastor.hide()'
	[[ ${systray_blink} == True ]] && echo 'SET@systray1.set_blinking(True)'

	ffmpeg_isRunnig=True

	# Lancement de la capture
	echo "TERM@@SEND@@${ffmepg_code}\n"

	sleep 0.5

	ffmpeg_pid=$(ps --ppid ${G2S_TERMINAL_PID} -o pid= 2>/dev/null)

	echo 'TIMER@@START@@500@@ffmpeg_timer'
fi
}

function ffmpeg_timer
{
# Fonction appelée par le TIMER
if [[ $(ps h -p ${ffmpeg_pid} 2>/dev/null) ]]
then
	ffmpeg_isRunnig=True

else
	ffmpeg_isRunnig=False

	if [[ ${flag_new_instance} == True ]]
	then
		# Une fois ffmpeg terminé, modifications graphiques
		echo 'SET@_menuitem_stop.set_sensitive(False)'
		echo 'SET@capture.show()'
		echo 'SET@_menuitem_record.set_sensitive(True)'
		echo 'SET@arret.hide()'
		[[ ${systray_auto} == True ]] && { echo 'SET@screencastor.set_visible(True)'; echo 'SET@screencastor.present()'; }
		[[ ${systray_blink} == True ]] && echo 'SET@systray1.set_blinking(False)'

		# Retour au dernier onglet connu
		[[ ${option_onglet} == True ]] && echo "SET@_notebook1.set_current_page(${onglet})"

		# Verifie que le fichier existe
		[[ -s "${sortie_fichier}" ]] && { echo 'SET@lecture.show()'; echo 'SET@_menuitem_play.set_sensitive(False)'; }

		_code_ffmpeg ${ffmepg_code}
	else
		echo 'TIMER@@STOP@@ffmpeg_timer'

		# Une fois ffmpeg terminé, modifications graphiques
		echo 'SET@_menuitem_stop.set_sensitive(False)'
		echo 'SET@capture.show()'
		echo 'SET@_menuitem_record.set_sensitive(True)'
		echo 'SET@arret.hide()'
		[[ ${systray_auto} == True ]] && { echo 'SET@screencastor.set_visible(True)'; echo 'SET@screencastor.present()'; }
		[[ ${systray_blink} == True ]] && echo 'SET@systray1.set_blinking(False)'

		# Retour au dernier onglet connu
		[[ ${option_onglet} == True ]] && echo "SET@_notebook1.set_current_page(${onglet})"

		# Verifie que le fichier existe
		[[ -s "${sortie_fichier}" ]] && { echo 'SET@lecture.show()'; echo 'SET@_menuitem_play.set_sensitive(False)'; }
	fi
fi
}

function arret
{
# Stop ffmpeg
kill -15 ${ffmpeg_pid}

sleep 0.5

# Lancement de la barre de progression qui indique ffmpeg n'a pas fini la compression
if [[ $(ps h -p ${ffmpeg_pid} 2>/dev/null) ]]
then
	echo 'SET@_fin_compression.show()'

	while [[ $(ps h -p ${ffmpeg_pid} 2>/dev/null) ]]
	do
		echo 'SET@_progression_compression.pulse()'
		sleep 0.1
	done

	echo 'SET@_fin_compression.hide()'
fi
}

function lecture { xdg-open "${sortie_fichier}"; }

function quit
{
if [[ ${1} == clic_droit ]]
then
	reload=yes
	echo 'SET@quit.activate()'

else
	# Si ffmpeg est en cours
	if [[ $(ps h -p ${ffmpeg_pid} 2>/dev/null) ]]
	then
		arret
		[[ -s "${sortie_fichier}" ]] && rm "${sortie_fichier}"
	fi

	[[ ${reload} == yes ]] && echo 'EXIT@@SAVE' || echo 'EXIT@@'
fi
}


###################
##### Options #####
###################
function pref_reload
{
if [[ ${1} == clic_droit ]]
then
	# Ouverture du fichier des preferences
	xdg-open "${fichier_pref}" &

else
	# Vérifie que le dossier des préférences existe
	[[ ! -d "${dossier_pref}" ]] && mkdir -p "${dossier_pref}"

	# Récupération de la resolution
	win_largeur=${G2S_SCREEN_WIDTH} win_hauteur=${G2S_SCREEN_HEIGHT}

	# Creation du nouveau fichier de préférence
	echo "langue=\"$(sed 's/_.*//' <<< ${LANG})\"
geo_fenetre=

### Preferences Video
video_no=\"False\"
video_codec=\"libx264\"

video_reso_l_out=\"${win_largeur}\"
video_reso_h_out=\"${win_hauteur}\"
video_reso_l_in=\"${win_largeur}\"
video_reso_h_in=\"${win_hauteur}\"
video_bitrate=\"700\"
video_tolerance=\"4000\"
video_qmin=\"10\"
video_qmax=\"50\"
video_desentralacement=\"False\"
video_multithreads=\"0\"
video_fps=\"30\"
video_framekey=\"250\"
video_framekey_min=\"25\"
video_treillis=\"1\"
video_deblockalpha=\"0\"
video_deblockbeta=\"0\"
video_bframe=\"16\"
video_references=\"0\"
video_lookahead=\"16\"
video_gris=\"False\"
video_debit=\"fixe\"
video_diasize=\"hex\"

x264_nocabac=\"False\"
x264_vpre=\"lossless_ultrafast\"
x264_crf=\"20\"
x264_algorithme=\"6\"

webm_level=\"116\"

### Preferences Audio
audio_no=\"False\"
audio_codec=\"libvorbis\"
audio_frequence=\"44100\"
audio_provenance=\"pulse\"
audio_bitrate=\"128\"
audio_serveur=\"alsa\"
audio_canal=\"2\"

### Preferences sortie
sortie_dossier=\"${HOME}\"
sortie_fps=\"25\"
sortie_display=\":0.0\"
sortie_emplacement=\"0,0\"
sortie_extension_audio=\"ogg\"
sortie_extension_video=\"mkv\"
capture_heure=\"0\"
capture_minute=\"0\"
capture_seconde=\"0\"
delay_heure=\"0\"
delay_minute=\"0\"
delay_seconde=\"0\"

### Preferences grahiques
systray_icone=\"True\"
systray_auto=\"False\"
systray_blink=\"False\"
option_onglet=\"False\"
unity=" > "${fichier_pref}"

	# Chargement des nouvelles valeurs
	source "${fichier_pref}"

	# Si la commande a été lancée depuis le glade
	if [[ ${1} == clicked ]]
	then
		# Commande pour eviter de recharger X fois le code
		lock=True

		# Dossier de sortie
		echo "SET@_sortie_dossier.set_current_folder('${sortie_dossier}')"

		# Chargement des préférence de type coche
		echo 'MULTI@@SET@@set_active(False)@@_systray_icone,_systray_auto,_systray_blink,_option_onglet,_video_gris,_x264_nocabac,_video_desentralacement'
		echo 'MULTI@@SET@@set_active(True)@@_debit_fixe'

		# Chargement de la langue
		case ${langue} in
			fr) echo 'COMBO@@FINDSELECT@@_langue@@Français' ;;
			en|*) echo 'COMBO@@FINDSELECT@@_langue@@English' ;;
		esac

		# Chargement des préférence de type combobox sans sortie_extension_video appellait par _x264
		( for i in audio_frequence audio_bitrate audio_provenance audio_serveur video_diasize x264_vpre webm_level sortie_extension_audio
		do
			echo "COMBO@@FINDSELECT@@_${i}@@${!i}"
		done )

		# Chargement des préférence de type set value si valeur differente de celle de base
		( for i in video_reso_l_out video_reso_h_out video_reso_l_in video_reso_h_in video_bitrate video_tolerance video_qmin video_qmax video_fps video_framekey video_framekey_min video_treillis video_deblockalpha video_deblockbeta video_bframe video_references video_lookahead x264_crf x264_algorithme sortie_fps capture_heure capture_minute capture_seconde delay_heure delay_minute delay_seconde video_multithreads
		do
			echo "SET@_${i}.set_value(${!i})"
		done )

		# Chargement des préférence de type entry
		echo "SET@_sortie_display.set_text('${sortie_display}')"
		echo "SET@_sortie_emplacement.set_text('${sortie_emplacement}')"

		# Tree
		echo "TREE@@FINDSELECT@@audio_codec@@0@@${audio_codec}"

		# Systray
		echo 'SET@systray1.set_visible(False)'

		# Chargement de l'onglet préféré lancant code_final à mettre ici pour que ça finisse par ça
		echo 'SET@_x264.set_active(True)'
		echo 'SET@_x264.toggled()'
	fi
fi
}


function pref_save
{
# Commandes demandant les valeurs
echo 'MULTI@@GET@@get_value()@@_video_reso_l_out,_video_reso_h_out,_video_reso_l_in,_video_reso_h_in,_video_bitrate,_video_tolerance,_video_qmin,_video_qmax,_video_references,_video_fps,_video_framekey,_video_framekey_min,_video_treillis,_video_deblockalpha,_video_deblockbeta,_video_bframe,_video_lookahead,_x264_crf,_x264_algorithme,_sortie_fps,_capture_heure,_capture_minute,_capture_seconde,_delay_heure,_delay_minute,_delay_seconde'

echo 'MULTI@@GET@@get_text()@@_sortie_display,_sortie_emplacement'

echo 'MULTI@@GET@@get_active()@@_systray_icone,_systray_auto,_systray_blink,_option_onglet'

# Lance la fonction suivante pour y récupérer les valeurs
echo 'ITER@@pref_save_suite'
}


function pref_save_suite
{
# Recuperation des variables
video_reso_l_out=${_video_reso_l_out_get_value} video_reso_l_out=${video_reso_l_out%.*}
video_reso_h_out=${_video_reso_h_out_get_value} video_reso_h_out=${video_reso_h_out%.*}
video_reso_l_in=${_video_reso_l_in_get_value} video_reso_l_in=${video_reso_l_in%.*}
video_reso_h_in=${_video_reso_h_in_get_value} video_reso_h_in=${video_reso_h_in%.*}
video_bitrate=${_video_bitrate_get_value} video_bitrate=${video_bitrate%.*}
video_tolerance=${_video_tolerance_get_value} video_tolerance=${video_tolerance%.*}
video_qmin=${_video_qmin_get_value} video_qmin=${video_qmin%.*}
video_qmax=${_video_qmax_get_value} video_qmax=${video_qmax%.*}
video_fps=${_video_fps_get_value} video_fps=${video_fps%.*}
video_framekey=${_video_framekey_get_value} video_framekey=${video_framekey%.*}
video_framekey_min=${_video_framekey_min_get_value} video_framekey_min=${video_framekey_min%.*}
video_treillis=${_video_treillis_get_value} video_treillis=${video_treillis%.*}
video_deblockalpha=${_video_deblockalpha_get_value} video_deblockalpha=${video_deblockalpha%.*}
video_deblockbeta=${_video_deblockbeta_get_value} video_deblockbeta=${video_deblockbeta%.*}
video_bframe=${_video_bframe_get_value} video_bframe=${video_bframe%.*}
video_references=${_video_references_get_value} video_references=${video_references%.*}
video_lookahead=${_video_lookahead_get_value} video_lookahead=${video_lookahead%.*}
x264_crf=${_x264_crf_get_value} x264_crf=${x264_crf%.*}
x264_algorithme=${_x264_algorithme_get_value} x264_algorithme=${x264_algorithme%.*}
sortie_fps=${_sortie_fps_get_value} sortie_fps=${sortie_fps%.*}
capture_heure=${_capture_heure_get_value} capture_heure=${capture_heure%.*}
capture_minute=${_capture_minute_get_value} capture_minute=${capture_minute%.*}
capture_seconde=${_capture_seconde_get_value} capture_seconde=${capture_seconde%.*}
delay_heure=${_delay_heure_get_value} delay_heure=${delay_heure%.*}
delay_minute=${_delay_minute_get_value} delay_minute=${delay_minute%.*}
delay_seconde=${_delay_seconde_get_value} delay_seconde=${delay_seconde%.*}

sortie_display=${_sortie_display_get_text}
sortie_emplacement=${_sortie_emplacement_get_text}

systray_icone=${_systray_icone_get_active}
systray_auto=${_systray_auto_get_active}
systray_blink=${_systray_blink_get_active}
option_onglet=${_option_onglet_get_active}

# Creation du fichier de preference avec les nouvelles valeurs
echo "langue=\"${langue}\"
geo_fenetre=\"${geo_fenetre}\"

### Preferences Video
video_no=\"${video_no}\"
video_codec=\"${video_codec}\"

video_reso_l_out=\"${video_reso_l_out}\"
video_reso_h_out=\"${video_reso_h_out}\"
video_reso_l_in=\"${video_reso_l_in}\"
video_reso_h_in=\"${video_reso_h_in}\"
video_bitrate=\"${video_bitrate}\"
video_tolerance=\"${video_tolerance}\"
video_qmin=\"${video_qmin}\"
video_qmax=\"${video_qmax}\"
video_desentralacement=\"${video_desentralacement}\"
video_multithreads=\"${video_multithreads}\"
video_fps=\"${video_fps}\"
video_framekey=\"${video_framekey}\"
video_framekey_min=\"${video_framekey_min}\"
video_treillis=\"${video_treillis}\"
video_deblockalpha=\"${video_deblockalpha}\"
video_deblockbeta=\"${video_deblockbeta}\"
video_bframe=\"${video_bframe}\"
video_references=\"${video_references}\"
video_lookahead=\"${video_lookahead}\"
video_gris=\"${video_gris}\"
video_debit=\"${video_debit}\"
video_diasize=\"${video_diasize}\"

x264_nocabac=\"${x264_nocabac}\"
x264_algorithme=\"${x264_algorithme}\"
x264_vpre=\"${x264_vpre}\"
x264_crf=\"${x264_crf}\"

webm_level=\"${webm_level}\"

### Preferences Audio
audio_no=${audio_no}
audio_codec=\"${audio_codec}\"
audio_frequence=\"${audio_frequence}\"
audio_provenance=\"${audio_provenance}\"
audio_bitrate=\"${audio_bitrate}\"
audio_serveur=\"${audio_serveur}\"
audio_canal=\"${audio_canal}\"

### Preferences sortie
sortie_dossier=\"${sortie_dossier}\"
sortie_fps=\"${sortie_fps}\"
sortie_display=\"${sortie_display}\"
sortie_emplacement=\"${sortie_emplacement}\"
sortie_extension_audio=\"${sortie_extension_audio}\"
sortie_extension_video=\"${sortie_extension_video}\"
capture_heure=\"${capture_heure}\"
capture_minute=\"${capture_minute}
capture_seconde=\"${capture_seconde}\"
delay_heure=\"${delay_heure}\"
delay_minute=\"${delay_minute}
delay_seconde=\"${delay_seconde}\"

### Preferences grahiques
systray_icone=\"${systray_icone}\"
systray_auto=\"${systray_auto}\"
systray_blink=\"${systray_blink}\"
option_onglet=\"${option_onglet}\"
unity=" > "${fichier_pref}"
}

function log_open { [[ -s "${fichier_log}" ]] && xdg-open "${fichier_log}"; }

function onglet { onglet=${_notebook1_get_current_page}; }



###########################
##### Systeme Systray #####
###########################
# Lors d'un clic gauche sur l'icone
function systray1
{
# Cache/Affiche la fenetre
if [[ ${etat:-visible} == visible ]]
then
	etat=cache
	echo 'SET@screencastor.set_visible(False)'
else
	etat=visible
	echo 'SET@screencastor.set_visible(True)'
	echo 'SET@screencastor.present()'
fi
}

function _systray_icone { systray_icone=${1}; echo "SET@systray1.set_visible(${1})"; }

function _systray_auto { systray_auto=${1}; }

function _systray_blink { systray_blink=${1}; }

function _option_onglet { option_onglet=${1}; }

function _unity
{
# Ajoute screencastor à la liste blanche
if [[ ${1} == True ]]
then
	if [[ $(gsettings get com.canonical.Unity.Panel systray-whitelist 2>/dev/null) ]]
	then
		# Récupération de la liste blanche
		OBJARRAY=$(gsettings get com.canonical.Unity.Panel systray-whitelist) OBJARRAY=${OBJARRAY/%\']} OBJARRAY=${OBJARRAY/#[\'} OBJARRAY=${OBJARRAY//\', \'/ }

		# Verifie que le logiciel n'est pas déjà dans la liste
		if [[ ! "${OBJARRAY}" =~ "screencastor.py" ]]
		then
			# Modification de la variable pour recréer une liste
			OBJARRAY+=" screencastor.py"
			OBJARRAY=${OBJARRAY// /\', \'}
			OBJSET="['"${OBJARRAY}"']"

			# Envoie de la commande
			gsettings set com.canonical.Unity.Panel systray-whitelist "${OBJSET}"

			# Reload d'unity
			unity &> /dev/null & disown
		fi
	fi

	# Modification du fichier preferences
	sed -i "s/unity=.*/unity=\"True\"/" "${fichier_pref}"

# Supprime screencastor de la liste blanche
elif [[ ${1} == False ]]
then
	if [[ $(gsettings get com.canonical.Unity.Panel systray-whitelist 2>/dev/null) ]]
	then
		# Récupération de la liste blanche
		OBJARRAY=$(gsettings get com.canonical.Unity.Panel systray-whitelist) OBJARRAY=${OBJARRAY/%\']} OBJARRAY=${OBJARRAY/#[\'} OBJARRAY=${OBJARRAY//\', \'/ }

		# Verifie que le logiciel est dans la liste
		if [[ "${OBJARRAY}" =~ "screencastor.py" ]]
		then
			# Modification de la variable pour recréer une liste
			OBJARRAY="${OBJARRAY// screencastor.py}"
			OBJARRAY=${OBJARRAY// /\', \'}
			OBJSET="['"${OBJARRAY}"']"

			# Envoie de la commande
			gsettings set com.canonical.Unity.Panel systray-whitelist "${OBJSET}"

			# Reload d'unity
			unity &> /dev/null & disown
		fi
	fi

	# Modification du fichier preferences
	sed -i "s/unity=.*/unity=/" "${fichier_pref}"
fi
}


#######################################
##### Chargement  des préférences #####
#######################################
function preferences
{
lock=True

# Dossier de sortie
[[ -z ${sortie_dossier} ]] && sortie_dossier="${HOME}"
echo "SET@_sortie_dossier.set_current_folder('${sortie_dossier}')"

# Langue du logiciel
case ${langue} in
	fr) echo 'COMBO@@FINDSELECT@@_langue@@Français' ;;
	en|*) echo 'COMBO@@FINDSELECT@@_langue@@English' ;;
esac

# Récupération de la resolution
win_largeur=${G2S_SCREEN_WIDTH} win_hauteur=${G2S_SCREEN_HEIGHT}
[[ -z ${video_reso_l_out} ]] && video_reso_l_out=${win_largeur}
[[ -z ${video_reso_h_out} ]] && video_reso_h_out=${win_hauteur}
[[ -z ${video_reso_l_in} ]] && video_reso_l_in=${win_largeur}
[[ -z ${video_reso_h_in} ]] && video_reso_h_in=${win_hauteur}

# Chargement des préférence de type coche
[[ ${video_no} == True ]] && echo 'SET@_video_no.set_active(True)'
[[ ${audio_no} == True ]] && echo 'SET@_audio_no.set_active(True)'
[[ ${video_desentralacement} == True ]] && echo 'SET@_video_desentralacement.set_active(True)'
[[ ${video_gris} == True ]] && echo 'SET@_video_gris.set_active(True)'
[[ ${x264_nocabac:-True} == True ]] && echo 'SET@_x264_nocabac.set_active(True)'
[[ ${option_onglet} == True ]] && echo 'SET@_option_onglet.set_active(True)'
[[ ${systray_icone} == True ]] && echo 'SET@_systray_icone.set_active(True)'
[[ ${systray_auto} == True ]] && echo 'SET@_systray_auto.set_active(True)'
[[ ${systray_blink} == True ]] && echo 'SET@_systray_blink.set_active(True)'

# Chargement des préférence de type combobox, ne pas mettre _sortie_extension_video car lancé via video_codec et c'est lui qui debloque le lock
echo "COMBO@@FINDSELECT@@_audio_frequence@@${audio_frequence:-44100}"
echo "COMBO@@FINDSELECT@@_audio_bitrate@@${audio_bitrate:-128}"
echo "COMBO@@FINDSELECT@@_audio_provenance@@${audio_provenance:-pulse}"
echo "COMBO@@FINDSELECT@@_audio_serveur@@${audio_serveur:-alsa}"
echo "COMBO@@FINDSELECT@@_video_diasize@@${video_diasize:-hex}"
echo "COMBO@@FINDSELECT@@_x264_vpre@@${x264_vpre:-lossless_ultrafast}"
echo "COMBO@@FINDSELECT@@_webm_level@@${webm_level:-116}"
echo "COMBO@@FINDSELECT@@_sortie_extension_audio@@${sortie_extension_audio:-ogg}"

# Chargement des préférence de type set value si valeur differente de celle de base
[[ ${video_multithreads:-0} -ne 0 ]] && echo "SET@_video_multithreads.set_value(${video_multithreads})"
[[ ${video_reso_l_out:-1024} -ne 1024 ]] && echo "SET@_video_reso_l_out.set_value(${video_reso_l_out})"
[[ ${video_reso_h_out:-768} -ne 768 ]] && echo "SET@_video_reso_h_out.set_value(${video_reso_h_out})"
[[ ${video_reso_l_in:-1024} -ne 1024 ]] && echo "SET@_video_reso_l_in.set_value(${video_reso_l_in})"
[[ ${video_reso_h_in:-768} -ne 768 ]] && echo "SET@_video_reso_h_in.set_value(${video_reso_h_in})"
[[ ${video_bitrate:-700} -ne 700 ]] && echo "SET@_video_bitrate.set_value(${video_bitrate})"
[[ ${video_tolerance:-4000} -ne 4000 ]] && echo "SET@_video_tolerance.set_value(${video_tolerance})"
[[ ${video_qmin:-10} -ne 10 ]] && echo "SET@_video_qmin.set_value(${video_qmin})"
[[ ${video_qmax:-50} -ne 50 ]] && echo "SET@_video_qmax.set_value(${video_qmax})"
[[ ${video_fps:-30} -ne 30 ]] && echo "SET@_video_fps.set_value(${video_fps})"
[[ ${video_framekey:-250} -ne 250 ]] && echo "SET@_video_framekey.set_value(${video_framekey})"
[[ ${video_framekey_min:-25} -ne 25 ]] && echo "SET@_video_framekey_min.set_value(${video_framekey_min})"
[[ ${video_treillis:-1} -ne 1 ]] && echo "SET@_video_treillis.set_value(${video_treillis})"
[[ ${video_deblockalpha:-0} -ne 0 ]] && echo "SET@_video_deblockalpha.set_value(${video_deblockalpha})"
[[ ${video_deblockbeta:-0} -ne 0 ]] && echo "SET@_video_deblockbeta.set_value(${video_deblockbeta})"
[[ ${video_bframe:-16} -ne 16 ]] && echo "SET@_video_bframe.set_value(${video_bframe})"
[[ ${video_references:-0} -ne 0 ]] && echo "SET@_video_references.set_value(${video_references})"
[[ ${video_lookahead:-16} -ne 16 ]] && echo "SET@_video_lookahead.set_value(${video_lookahead})"
[[ ${x264_crf:-20} -ne 20 ]] && echo "SET@_x264_crf.set_value(${x264_crf})"
[[ ${x264_algorithme:-6} -ne 6 ]] && echo "SET@_x264_algorithme.set_value(${x264_algorithme})"
[[ ${audio_canal:-2} -ne 2 ]] && echo "SET@_audio_canal.set_value(${audio_canal})"
[[ ${sortie_fps:-25} -ne 25 ]] && echo "SET@_sortie_fps.set_value(${sortie_fps})"
[[ ${capture_heure:-0} -ne 0 ]] && echo "SET@_capture_heure.set_value(${capture_heure})"
[[ ${capture_minute:-0} -ne 0 ]] && echo "SET@_capture_minute.set_value(${capture_minute})"
[[ ${capture_seconde:-0} -ne 0 ]] && echo "SET@_capture_seconde.set_value(${capture_seconde})"
[[ ${delay_heure:-0} -ne 0 ]] && echo "SET@_delay_heure.set_value(${delay_heure})"
[[ ${delay_minute:-0} -ne 0 ]] && echo "SET@_delay_minute.set_value(${delay_minute})"
[[ ${delay_seconde:-0} -ne 0 ]] && echo "SET@_delay_seconde.set_value(${delay_seconde})"

# Chargement des préférence de type entry, avec des choix par defaut si besoin
[[ ${sortie_display} != ":0.0" && ${sortie_display} ]] && echo "SET@_sortie_display.set_text('${sortie_display}')"
[[ ${sortie_emplacement} != "0,0" &&  ${sortie_emplacement} ]] && echo "SET@_sortie_emplacement.set_text('${sortie_emplacement}')"

# Tree
echo "TREE@@FINDSELECT@@audio_codec@@0@@${audio_codec:-vorbis}"

# Chargement du type de debit
case ${video_debit:-fixe} in
	fixe) echo 'SET@_debit_fixe.set_active(True)'; echo 'SET@_debit_fixe.toggled()' ;;
	variable) echo 'SET@_debit_variable.set_active(True)';;
esac

# Systray
[[ ${systray_icone} == True ]] && echo 'SET@systray1.set_visible(True)'
[[ ${unity} == True ]] && echo 'SET@_unity.set_visible(True)'

# Chargement de l'onglet préféré lancant code_final à mettre ici pour que ça finisse par ça
if [[ ${video_no} != True ]]
then
	case ${video_codec} in
		libvpx) echo 'SET@_webm.set_active(True)' ;;
		flv) echo 'SET@_flash.set_active(True)' ;;
		libx264|*) echo 'SET@_x264.set_active(True)'; echo 'SET@_x264.toggled()' ;;
		# Necessite cette 2eme commande pour executer x264 actif de base
	esac
# Utile dans le cas ou pas de video sauvegardé
else
	code_final
fi
}

function _langue
{
# Récupération de la bonne valeur de langue en fonction du choix
case ${@} in
	Français) langue=fr ;;
	English) langue=en ;;
esac

# Modification de langue dans les preferences
sed -i "s@^langue=.*@langue=\"${langue}\"@" "${fichier_pref}"
}

function langue_reload
{
# Redemarrage du logiciel
quit clic_droit
}


########################
##### Introduction #####
########################
version='1.0.2'
dossier_pref="${HOME}/.config/screencastor"
fichier_pref="${dossier_pref}/preferences"
ffmpeg_audio="${dossier_pref}/ffmpeg_audio"
fichier_log="${dossier_pref}/Screencastor.log"
ffmpeg_isRunnig=false
ffmpeg_pid=
flag_new_instance=false
visible=False

# Chargement des variables préférences ou création du fichier s'il n'existe pas
[[ -f "${fichier_pref}" ]] && source "${fichier_pref}" || pref_reload

## Deplacement de la fenetre
[[ ${geo_fenetre} ]] && echo "GEO@@SET@@screencastor@@${geo_fenetre}"

# Mise en place des commandes pour le terminal
echo 'SET@terminal.set_font( pango.FontDescription("monospace 8") )'
echo 'SET@terminal.set_scrollback_lines(100)'

# Commande de creation du systray
echo 'SYSTRAY@@systray1@@menu1@@gtk-yes.png/usr/share/icons/hicolor/256x256/apps/screencastor.png@@Screencastor'

# Gestion du fichier log enregistrant tout probleme
[[ -e "${fichier_log}" ]] && rm "${fichier_log}"
exec 2>> "${fichier_log}"

# Textview : coloration et saut automatique de ligne
echo "TEXT@@SOURCE@@LANG@@_code_ffmpeg@@sh"
echo 'SET@_code_ffmpeg.set_wrap_mode(gtk.WRAP_WORD)'

# Modifications graphiques
echo "SET@Screencastor.set_version('v${version}')"
echo "SET@screencastor.set_title('Screencastor v${version} - by Hizoka')"

# Si le fichier liste des codecs audio n'existe pas, on le crée
[[ ! -f "${ffmpeg_audio}" ]] && audio_liste || echo "TREE@@LOAD@@audio_codec@@${ffmpeg_audio}"

# Chargement des preferences
preferences

######################################################################################################

while read ligne
do
	if [[ ${ligne} =~ GET@ ]]
	then
		eval ${ligne#*@}
		echo "DEBUG => in boucle bash :" ${ligne#*@}
	else
		echo "DEBUG=> in bash NOT GET" ${ligne}
		${ligne}
	fi
done < <(while true
do
	read entree < ${FIFO}
	[[ ${entree} == QuitNow ]] && break
	echo ${entree}
done)


exit
